﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.ServiceModel
Imports PeerWcfComm

Namespace LoggerPeerApplication
	''' <summary>
	''' The logging peer.  Participates in IWorker and ISpectator communications.
	''' It simply logs to standard output.
	''' </summary>
	<ServiceBehavior(InstanceContextMode := InstanceContextMode.Single)> _
	Public Class Logger
		Implements IWorker, ISpectator
		Private _host As ServiceHost
		Private _workers As Dictionary(Of Guid, WorkerStatus)

		Public Sub New()
			_workers = New Dictionary(Of Guid, WorkerStatus)()
		End Sub

		Public Sub Start()
			_host = New ServiceHost(Me)
			_host.Open()
		End Sub

		Public Sub [Stop]()
			_host.Close()
		End Sub

		#Region "IWorker Members"

		Private Sub Heartbeat(ByVal status As WorkerStatus) Implements IWorker.Heartbeat
			'Only log a change in state.
			Dim changed As Boolean = False

			SyncLock _workers
				If _workers.ContainsKey(status.WorkerId) Then
					Dim s As WorkerStatus = _workers(status.WorkerId)

					If s.State <> status.State Then
						changed = True
					End If
				Else
					changed = True
				End If

				_workers(status.WorkerId) = status
			End SyncLock

			If changed Then
				LogWorkerStatusChange(status)
			End If
		End Sub

		Private Sub DoWork(ByVal workerId As Guid, ByVal work As WorkItem) Implements IWorker.DoWork
			'Ignored. 
			'This can be assumed by the WorkItemState.Queued status event.
		End Sub
		#End Region

		#Region "ISpectator Members"

		Private Sub WorkItemStateChanged(ByVal status As WorkItemStatus) Implements ISpectator.WorkItemStateChanged
			Console.WriteLine("Worker: {0}| WorkItem: {1}| State: {2}", status.WorkerId, status.WorkItemId, status.NewState)
		End Sub

		#End Region

		Private Sub LogWorkerStatusChange(ByVal status As WorkerStatus)
			Console.WriteLine("Worker {0}| State: {1}", status.WorkerId, status.State)
		End Sub
	End Class
End Namespace