﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.ServiceModel
Imports System.Windows.Forms
Imports PeerWcfComm

Namespace UserPeerApplication
	''' <summary>
	''' Definition of both the User peer and the main form
	''' for this application. This peer participates in IWorker
	''' communication.
	''' </summary>
	<ServiceBehavior(InstanceContextMode := InstanceContextMode.Single)> _
	Partial Public Class User
		Inherits Form
		Implements IWorker
		Private _host As ServiceHost
		Private _factory As ChannelFactory(Of IWorker)
		Private _clientChannel As IWorker
		Private _workers As Dictionary(Of Guid, WorkerStatus)
		Private _rnd As Random

		Public Sub New()
			InitializeComponent()

			_rnd = New Random(DateTime.Now.Millisecond)
			_workers = New Dictionary(Of Guid, WorkerStatus)()

			_host = New ServiceHost(Me)
			_host.Open()

			_factory = New ChannelFactory(Of IWorker)("IWorkerEndpoint")
			_clientChannel = _factory.CreateChannel()

			Dim pn As PeerNode = (CType(_clientChannel, IClientChannel)).GetProperty(Of PeerNode)()
			pn.MessagePropagationFilter = New RemoteOnlyMessagePropagationFilter()
		End Sub

		#Region "IWorker Members"

		Private Sub Heartbeat(ByVal status As WorkerStatus) Implements IWorker.Heartbeat
			SyncLock _workers
				_workers(status.WorkerId) = status
			End SyncLock
		End Sub

		Private Sub DoWork(ByVal workerId As Guid, ByVal work As WorkItem) Implements IWorker.DoWork
			'This doesn't make sense for a user peer.
		End Sub
		#End Region

		''' <summary>
		''' Tick event handler for our form timer. This will update
		''' the UI according to our latest state.
		''' </summary>
		Private Sub timer1_Tick(ByVal sender As Object, ByVal e As EventArgs) Handles timer1.Tick
			Me.lstWorkers.Items.Clear()

			'Copy the Ids of our workers to a temp variable.
			Dim workers As List(Of Guid)
			SyncLock _workers
				workers = New List(Of Guid)(_workers.Keys)
			End SyncLock

			'Loop through the list, get the WorkerStatus,
			'and create a ListViewItem for our UI.
			For Each worker As Guid In workers
				Dim status As WorkerStatus

				SyncLock _workers
					status = _workers(worker)
				End SyncLock

				Me.lstWorkers.Items.Add(New ListViewItem(New String() { worker.ToString(), status.State.ToString(), status.CurrentWorkItem.ToString(), status.PendingWorkItemCount.ToString() }))
			Next worker
		End Sub

		Private Sub btnNewWork_Click(ByVal sender As Object, ByVal e As EventArgs) Handles btnNewWork.Click
			'Create a new WorkItem with random Id and a
			'"work" period from 0 to 15 seconds.
			Dim wrkItem As New WorkItem()
			wrkItem.WorkItemId = Guid.NewGuid()
			wrkItem.PretendToWorkSeconds = _rnd.Next(15)

			'Get the next worker that "needs" work.
			Dim nextWorker As WorkerStatus = GetNextWorker()

			If nextWorker IsNot Nothing Then
				_clientChannel.DoWork(nextWorker.WorkerId, wrkItem)

				'Add onto the state, to account for the work just sent.
				'This gives increased accuracy since it could be a while
				'to get an updated heartbeat from this worker with the
				'updated count.
				SyncLock _workers
					_workers(nextWorker.WorkerId).PendingWorkItemCount += 1
				End SyncLock
			End If
		End Sub

		Private Function GetNextWorker() As WorkerStatus
			'Quickly get the WorkerStatus values in our dictionary.
			Dim workers As New List(Of WorkerStatus)()
			SyncLock _workers
				workers.AddRange(_workers.Values)
			End SyncLock

			'Eliminate offline and unknown Workers
			'and sort by the PendingWorkItemCount.
			'Ideally something like a min heap would be a
			'better data structure.  But this is more than
			'sufficient for now.
            Return workers _
                .Where(Function(w) (w.State <> WorkerState.Offline AndAlso w.State <> WorkerState.Unknown)) _
                .OrderBy(Function(w) w.PendingWorkItemCount).FirstOrDefault()
		End Function
	End Class
End Namespace
